from radicalbit_ai_monitoring import MODULE_PATH
from radicalbit_ai_monitoring import DEFAULT_VERSION, ALL_IMAGES, RBIT_IMAGES
from radicalbit_ai_monitoring.models.compose_service import ComposeService

from os import path, makedirs
from shutil import copy2, copytree, rmtree
import yaml

from typing import List


class FSUtil:
    @staticmethod
    def create_project_structure(dir: str, compose_file: str) -> None:
        resources_path = path.abspath(path.join(MODULE_PATH, "resources"))
        docker_resources_path = path.abspath(path.join(resources_path, "docker"))
        compose_path = path.abspath(path.join(resources_path, compose_file))

        makedirs(dir)
        # copy in the current directory
        copy2(compose_path, path.join(dir, "docker-compose.yaml"))
        copytree(docker_resources_path, path.join(dir, "docker"))

    @staticmethod
    def remove_project_structure(dir: str) -> None:
        rmtree(dir, ignore_errors=False)

    @staticmethod
    def installed(dir: str) -> bool:
        return path.exists(dir)

    @staticmethod
    def get_versions(dir: str) -> List[ComposeService]:
        compose_path = path.abspath(path.join(dir, "docker-compose.yaml"))
        with open(compose_path) as f:
            compose_config = yaml.safe_load(f)
        return [
            ComposeService(
                name=service,
                image=compose_config["services"][service]["image"].split(":")[0],
                tag=compose_config["services"][service]["image"].split(":")[1],
            )
            for service in ALL_IMAGES.keys()
        ]

    @staticmethod
    def update_compose_file_version(
        dir: str,
        compose_file: str = "docker-compose.yaml",
        version: str = DEFAULT_VERSION,
    ) -> None:
        compose_path = path.abspath(path.join(dir, compose_file))
        with open(compose_path) as f:
            compose_config = yaml.safe_load(f)

        # update version
        for service, image in RBIT_IMAGES.items():
            compose_config["services"][service]["image"] = f"{image}:{version}"

        with open(compose_path, "w") as f:
            yaml.dump(compose_config, f)



import click
import pyfiglet

from radicalbit_ai_monitoring import CLI_VERSION, OSS_REPO, RADICALBIT_FIGLET


@click.command()
def info():
    """Show Info About Radicalbit AI"""
    message = pyfiglet.figlet_format(RADICALBIT_FIGLET, font="slant")
    click.echo(message)
    click.echo(f"CLI Version: {CLI_VERSION}")
    click.echo("Author: https://github.com/radicalbit")
    click.echo(f"Radicalbit AI Monitoring GitHub: {OSS_REPO}")



import click

from radicalbit_ai_monitoring.commands.platform.platform import platform
from radicalbit_ai_monitoring.commands.info import info
from radicalbit_ai_monitoring import CLI_VERSION


@click.group()
@click.version_option(version=CLI_VERSION, message="%(prog)s %(version)s")
def cli():
    pass


cli.add_command(platform)
cli.add_command(info)



import click

from radicalbit_ai_monitoring import DEFAULT_VERSION, AVAILABLE_VERSIONS
from radicalbit_ai_monitoring.services.docker import DockerUtil
from radicalbit_ai_monitoring.services.fs import FSUtil

from pathlib import Path


@click.command()
@click.option(
    "--dir",
    "-d",
    default=f"{Path.home()}/radicalbit-ai-monitoring",
    help="The name of the directory to create",
    show_default=True,
)
@click.option(
    "--version",
    "-v",
    default=DEFAULT_VERSION,
    help="The version of the docker image to use",
    show_default=True,
    type=click.Choice(AVAILABLE_VERSIONS),
)
@click.option(
    "--overwrite",
    "-o",
    is_flag=True,
    default=False,
    help="Overwrite the current installation",
    show_default=True,
)
def install(dir: str, version: str, overwrite: bool = False):
    """Install the radicalbit-ai-monitoring"""
    click.echo(f"Installing radicalbit-ai-monitoring {version}")
    # Check if docker is installed and up and running
    docker = DockerUtil(dir)
    docker.dependencies_check()

    if FSUtil.installed(dir) or overwrite:
        if click.confirm(
            f"Found an existing installation in {dir}.\nDo you want to overwrite it?",
            abort=True,
        ):
            docker.clean_environment()
            FSUtil.remove_project_structure(dir)

    FSUtil.create_project_structure(dir, "docker-compose-local.yaml")

    if version != DEFAULT_VERSION:
        FSUtil.update_compose_file_version(dir, version=version)

    # run the compose file
    docker.run_docker_compose_down()
    versions = FSUtil.get_versions(dir)
    pull = docker.need_pull(versions)
    docker.run_app(pull)

    click.echo("\nRadicalbit-ai-monitoring started successfully")
    # TODO: make the port configurable and update the docker-compose file
    click.echo("You can access it at http://127.0.0.1:5173")



import click

from radicalbit_ai_monitoring.services.docker import DockerUtil
from radicalbit_ai_monitoring.services.fs import FSUtil

from pathlib import Path

import sys


@click.command(help="Install the radicalbit-ai-monitoring")
@click.option(
    "--dir",
    "-d",
    default=f"{Path.home()}/radicalbit-ai-monitoring",
    help="Path of the installation",
    show_default=True,
)
def up(dir: str):
    """Run the radicalbit-ai-monitoring platform"""
    if not FSUtil.installed(dir):
        click.echo("Radicalbit-ai-monitoring is not installed")
        click.echo("Run 'radicalbit-ai-monitoring install --help' for more info")
        sys.exit(1)

    click.echo("Starting radicalbit-ai-monitoring...")
    docker = DockerUtil(dir)

    versions = FSUtil.get_versions(dir)
    pull = docker.need_pull(versions)
    docker.run_app(pull)

    click.echo("Radicalbit-ai-monitoring started successfully")
    # TODO: make the port configurable and update the docker-compose file
    click.echo("You can access it at http://127.0.0.1:5173")



import os.path as path
import radicalbit_ai_monitoring


# x-release-please-start-version
CLI_VERSION = "0.7.0"
# x-release-please-end
MODULE_PATH = path.abspath(path.dirname(radicalbit_ai_monitoring.__file__))

RBIT_IMAGES = {
    "api": "radicalbit/radicalbit-ai-monitoring-api",
    "ui": "radicalbit/radicalbit-ai-monitoring-ui",
    "migrations": "radicalbit/radicalbit-ai-monitoring-migrations",
}

EXTERNAL_IMAGES = {
    "postgres": "postgres:15.7-alpine",
    "minio": "minio/minio:latest",
    "createbuckets": "minio/mc:latest",
    "init-data": "postgres:15.7-alpine",
    "minio-mirror": "minio/minio:latest",
    "k3s": "rancher/k3s:v1.30.1-k3s1",
}

ALL_IMAGES = {**RBIT_IMAGES, **EXTERNAL_IMAGES}

DEFAULT_VERSION = "latest"

OSS_REPO = "https://github.com/radicalbit/radicalbit-ai-monitoring"
OSS_REPO_API = "https://api.github.com/repos/radicalbit/radicalbit-ai-monitoring"

AVAILABLE_VERSIONS = ["latest", "v1.0.0"]

RADICALBIT_FIGLET = "Radicalbit AI Monitoring"



import click

from radicalbit_ai_monitoring.services.docker import DockerUtil
from radicalbit_ai_monitoring.services.fs import FSUtil

from pathlib import Path
import sys


@click.command()
@click.option(
    "--dir",
    "-d",
    default=f"{Path.home()}/radicalbit-ai-monitoring",
    help="Path of the installation",
    show_default=True,
)
@click.option(
    "--verbose",
    "-v",
    is_flag=True,
    default=False,
    help="Enable verbose mode",
    show_default=True,
)
@click.confirmation_option(
    prompt="Are you sure you want to uninstall radicalbit-ai-monitoring?",
    help="Confirm that you want to uninstall",
)
def uninstall(dir: str, verbose: bool = False):
    """Uninstall the radicalbit-ai-monitoring"""
    if not FSUtil.installed(dir):
        click.echo("Radicalbit-ai-monitoring is not installed. Nothing to do")
        sys.exit(1)
    docker = DockerUtil(dir)

    click.echo("Uninstalling radicalbit-ai-monitoring...")

    docker.clean_environment(verbose=verbose)
    FSUtil.remove_project_structure(dir)

    click.echo("Radicalbit-ai-monitoring uninstalled successfully")



from pydantic import BaseModel

from typing import Optional


class ComposeService(BaseModel):
    name: str
    image: str
    tag: str
    ports: Optional[list] = None
    volumes: Optional[list] = None
    environment: Optional[list] = None
    command: Optional[list] = None



import click

from radicalbit_ai_monitoring import RADICALBIT_FIGLET
from radicalbit_ai_monitoring.commands.platform.install import install
from radicalbit_ai_monitoring.commands.platform.stop import down
from radicalbit_ai_monitoring.commands.platform.uninstall import uninstall
from radicalbit_ai_monitoring.commands.platform.up import up

import pyfiglet


@click.group()
def platform():
    """Radicalbit AI Monitoring Platform utilities"""
    message = pyfiglet.figlet_format(RADICALBIT_FIGLET, font="slant")
    click.echo(message)


platform.add_command(install)
platform.add_command(down)
platform.add_command(uninstall)
platform.add_command(up)



import subprocess
from tempfile import TemporaryFile
from typing import List

from radicalbit_ai_monitoring.models.compose_service import ComposeService
from radicalbit_ai_monitoring.models.docker_status import DockerStatus
import json
from os import path
from time import sleep
import sys

import click

from yaspin import yaspin


class DockerUtil:
    def __init__(self, dir: str) -> None:
        self.dir = dir

    @staticmethod
    def dependencies_check() -> None:
        match DockerUtil.check_docker_installation():
            case DockerStatus.DOCKER_NOT_INSTALLED:
                click.echo("Docker can't be found")
                sys.exit(1)
            case DockerStatus.DOWN:
                click.echo("Docker daemon is not running")
                sys.exit(1)

        match DockerUtil.check_docker_compose_installation():
            case DockerStatus.DOCKER_COMPOSE_NOT_INSTALLED:
                click.echo("Docker Compose can't be found")
                sys.exit(1)

    @staticmethod
    def check_docker_installation() -> DockerStatus:
        try:
            ret = subprocess.run(
                ["docker", "version"], stdout=subprocess.PIPE, stderr=subprocess.PIPE
            )
        except OSError as e:
            from errno import ENOENT

            if e.errno == ENOENT:
                return DockerStatus.DOCKER_NOT_INSTALLED
        if ret.stderr.startswith(b"Cannot connect to the Docker daemon"):
            return DockerStatus.DOWN
        return DockerStatus.UP

    @staticmethod
    def check_docker_compose_installation() -> DockerStatus:
        try:
            subprocess.run(
                ["docker", "compose", "version"],
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
            )
        except OSError as e:
            from errno import ENOENT

            if e.errno == ENOENT:
                return DockerStatus.DOCKER_COMPOSE_NOT_INSTALLED
        return DockerStatus.UP

    def run_docker_compose_up(self) -> None:
        subprocess.Popen(
            ["docker", "compose", "up", "-d"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            cwd=path.abspath(self.dir),
        )
        self.wait_for_containers_ready()

    def run_docker_compose_down(self, *args, verbose: bool = False) -> None:
        with yaspin(text="Stopping the app", color="blue") as spinner:
            p = subprocess.Popen(
                ["docker", "compose", "down", *args],
                cwd=path.abspath(self.dir),
                stdout=None if verbose else subprocess.DEVNULL,
                stderr=None if verbose else subprocess.DEVNULL,
            )
            p.wait()

            if p.returncode != 0:
                spinner.fail("💥")
                click.echo(
                    f"\nError while stopping containers:\n{p.stderr.read().decode()}"
                )
                sys.exit(1)

            spinner.ok("✔")

    def clean_environment(self, verbose: bool = False) -> None:
        self.run_docker_compose_down("-v", "--remove-orphans", verbose=verbose)

    def need_pull(self, services: List[ComposeService]) -> bool:
        p = subprocess.Popen(
            ["docker", "image", "ls", "--format", "{{.Repository}}:{{.Tag}}"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            cwd=path.abspath(self.dir),
        )
        p.wait()

        if p.returncode != 0:
            click.echo(f"\nError while checking images:\n{p.stderr.read().decode()}")
            sys.exit(1)

        res = p.stdout.read().decode().split("\n")
        return any(f"{s.image}:{s.tag}" not in res for s in services)

    def run_docker_compose_pull(self) -> None:
        with TemporaryFile() as f, yaspin(
            text="Pulling images. This can take a while", color="blue"
        ) as spinner:
            pull_status_proc = subprocess.Popen(
                ["docker", "compose", "pull"],
                stdout=f,
                stderr=f,
                cwd=path.abspath(self.dir),
            )
            while pull_status_proc.poll() is None:
                sleep(0.5)

            if pull_status_proc.returncode != 0:
                spinner.fail("💥")
                click.echo("\nError while pulling images")
                sys.exit(1)

            spinner.ok("✔")

    def wait_for_containers_ready(self) -> None:
        with yaspin(text="Wait for application startup", color="blue") as spinner:
            ps_cmd = ["docker", "compose", "ps", "--all", "--format", "json"]
            while True:
                docker_ps_result = subprocess.run(
                    ps_cmd,
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE,
                    cwd=path.abspath(self.dir),
                )
                out = docker_ps_result.stdout.decode("utf-8").split("\n")
                out = filter(lambda x: x != "", out)
                json_res = [json.loads(svc) for svc in out]
                if len(json_res) > 0 and all(
                    [
                        svc["State"] == "running" or svc["State"] == "exited"
                        for svc in json_res
                    ]
                ):
                    break
            spinner.ok("✔")

    def run_app(self, pull: bool = True) -> None:
        if pull:
            self.run_docker_compose_pull()
        self.run_docker_compose_up()



import click

from pathlib import Path

from radicalbit_ai_monitoring.services.docker import DockerUtil
from radicalbit_ai_monitoring.services.fs import FSUtil
import sys


@click.command(help="Stop radicalbit-ai-monitoring")
@click.option(
    "--clean",
    "-c",
    is_flag=True,
    default=False,
    help="Clear the environment volumes",
    show_default=True,
)
@click.option(
    "--verbose",
    "-v",
    is_flag=True,
    default=False,
    help="Enable verbose mode",
    show_default=True,
)
@click.option(
    "--dir",
    "-d",
    default=f"{Path.home()}/radicalbit-ai-monitoring",
    help="Path of the installation",
    show_default=True,
)
def down(clean: bool, dir: str, verbose: bool = False):
    """Stop radicalbit-ai-monitoring"""
    if not FSUtil.installed(dir):
        click.echo("Radicalbit-ai-monitoring is not installed")
        click.echo("Run 'radicalbit-ai-monitoring install --help' for more info")
        sys.exit(1)
    docker = DockerUtil(dir)

    click.echo("Stopping radicalbit-ai-monitoring...")

    if clean and click.confirm(
        "Are you sure you want to clear the environment?", abort=False
    ):
        docker.clean_environment(verbose=verbose)
        click.echo("Radicalbit-ai-monitoring environment cleared successfully")
    else:
        docker.run_docker_compose_down(verbose=verbose)

    click.echo("Radicalbit-ai-monitoring stopped successfully")
